/* -*-C++-*-
 * ###################################################################
 *  Cpptcl - connecting C++ with Tcl
 * 
 *  FILE: "member_matrix.h"
 *                                    created: 29/10/97 {9:25:59 am} 
 *                                last update: 20/4/98 {6:03:49 pm} 
 *  Author: Vince Darley
 *  E-mail: <darley@fas.harvard.edu>
 *    mail: Division of Engineering and Applied Sciences, Harvard University
 *          Oxford Street, Cambridge MA 02138, USA
 *     www: <http://www.fas.harvard.edu/~darley/>
 *  
 * ========================================================================
 *               Copyright (c) 1997 Vince Darley
 * ========================================================================
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that the copyright notice and warranty disclaimer appear in
 * supporting documentation.
 * 
 * Vince Darley disclaims all warranties with regard to this software,
 * including all implied warranties of merchantability and fitness.  In
 * no event shall Vince Darley be liable for any special, indirect or
 * consequential damages or any damages whatsoever resulting from loss of
 * use, data or profits, whether in an action of contract, negligence or
 * other tortuous action, arising out of or in connection with the use or
 * performance of this software.
 * ========================================================================
 *  Description: 
 * 
 *  History
 * 
 *  modified by  rev reason
 *  -------- --- --- -----------
 *  29/10/97 VMD 1.0 original
 * ###################################################################
 */

#ifndef _Cpptcl_member_matrix_
#define _Cpptcl_member_matrix_

#include "cpptcl_config_mem.h"

//@Section: Cpptcl library
//@Man: 
class mem_matrix : public cpp_config_mem {
  protected:
	mem_matrix(const char* n):cpp_config_mem(n) {}	
	mem_matrix(const mem_matrix& m):cpp_config_mem(m) {}	
  public:
	Cpptcl_BaseClass(mem_matrix);
	~mem_matrix(void) {}
    ///
	void tcl_get(tcl_object* o, tcl_obj& tcl_) const;
    ///
	void tcl_set(tcl_object* o, tcl_args_reader& t) const;
	///
	void conversion_error(tcl_args& arg) const;
	///
	void get_syntax(tcl_args& arg) const;
	///
	int parse_meta_commands(tcl_obj& tcl_, tcl_args& arg);
	
};

class wrapped_matrix {
  public:  
	virtual int dimensions(void) const=0;
	virtual int dimension(int) const=0;
	int initialise(const Tcl_Obj*& obj) {
		return initialise(0,0,1,&obj);
	}
	int output(tcl_obj& ob) const {
		return output(0,0,1,ob);
	}
	tcl_args_reader& operator>> (tcl_args_reader& a) {
		Tcl_Obj* ob =0;
		a >> ob;
		initialise(ob);
		return a;
	}
	tcl_obj& operator<< (tcl_obj& ob) {
		output(ob);
		return ob;
	}
	virtual int set_range(int newoffs, int nargs, const Tcl_Obj** obj)=0;
	virtual int get_range(int newoffs, int nargs, tcl_obj& obj) const=0;
  protected:
	virtual void * void_ptr(void)=0;
	int initialise(int dim, int offs, int nargs, const Tcl_Obj** obj);
	int output(int dim, int offs, int nargs, tcl_obj& obj) const;
};


template <class T>
class wrapped_matrix_t :  public wrapped_matrix {
  public:
	virtual T* ptr(void)=0;
	virtual const T* ptr(void) const=0;
	void * void_ptr(void) {
		return (void*) ptr();
	}
	int set_range(int newoffs, int nargs, const Tcl_Obj** obj);
	int get_range(int newoffs, int nargs, tcl_obj& obj) const;

};

//@Section: Cpptcl library
//@Man: 
template <class T> class member_matrix: public mem_matrix {
  protected:
	///
	cpp_mem* copy(void) const {return new member_matrix<T>(*this);}
	///
	member_matrix(const member_matrix<T>& c):mem_matrix(c),_mptr(c._mptr) {}
	///
	wrapped_matrix_t<T> tcl_object::*_mptr;
  public:  
	///
	static const char* _data_type;
	///
	const char* data_type(void) const { return _data_type;}
	///
	T* ptr(tcl_object* o) { return reference().ptr;}
	///
	wrapped_matrix_t<T>& reference(tcl_object* o) const { return o->*_mptr;}
	///
	wrapped_matrix_t<T>& get(tcl_object * o) const { return o->*_mptr;}
	///
	wrapped_matrix_t<T>& set(tcl_object * o, const a_matrix<T>& val) const { return (o->*_mptr = val);}	
	///
	member_matrix(const char* n, a_matrix<T> tcl_object::*ptr):mem_matrix(n),_mptr(ptr) {}
	member_matrix(const char* n, a_matrix<T> tcl_object::*ptr):mem_matrix(n),_mptr(ptr) {}
	~member_matrix(void) {}
	///
	virtual bool private_data(const cpp_private& data) const {
		return (data == (cpp_mptr)_mptr);
	}
	void tcl_get(tcl_object* o, tcl_obj& tcl) const;
	
	void tcl_set(tcl_object* o, tcl_args_reader& istr) const;
};

#define cppMatrixmember(n,a,T) \
	matrix_member<T>(n,(a_matrix<T> tcl_object::*)&a)
#define cppMemMatrixType(T) const char* member_matrix<T>::_data_type = #T "-matrix"

#endif
