/* -*-C++-*-
 * ###################################################################
 * 
 *	FILE: "pltcl_stream.cc"
 *									  created: 19/7/96 {5:01:47 pm}	
 *								  last update: 04/02/1998 {09:03:25 AM}		
 *	Author:	Vince Darley
 *	E-mail:	<vince@das.harvard.edu>
 *	  mail:	Division of	Applied	Sciences, Harvard University
 *			Oxford Street, Cambridge MA	02138, USA
 *	   www:	<http://www.fas.harvard.edu/~darley/>
 *	
 *	See	header file	for	further	information
 * ###################################################################
 */


#include "pltcl_stream.h"
#include "plcpp_args.h"
#define PLPLOT_MATRIX_CLASSES
#include "tclMatrix.h"

static int tclmateval_modx,	tclmateval_mody;
PLFLT tclMatrix_feval (PLINT i,	PLINT j, PLPointer p);

PLFLT tclMatrix_feval (PLINT i,	PLINT j, PLPointer p)
{
	tclMatrix *matPtr =	(tclMatrix *) p;

	i =	i %	tclmateval_modx;
	j =	j %	tclmateval_mody;

/*	  printf( "tclMatrix_feval:	i=%d j=%d f=%f\n", i, j,
	  matPtr->fdata[I2D(i,j)] );
			*/
	return matPtr->fdata[I2D(i,j)];
}

Cpptcl_IClassNoMembers(pltcl_stream,"pltcl",tcl_object);

pltcl_stream::pltcl_stream(tcl_args& arg)
	:plstream(PLS::Current),tcl_object(arg)
{
}

int pltcl_stream::parse_tcl_command(tcl_args& arg){
#if 0
	//#include "pltcl_parser.cpp"
#else
if(arg("?page?",
		          "Advance to subpage \"page\", or to the next one if \"page\" = 0.")
		      =="adv") {
		PLINT  page =  0;
		arg >> optional >> page;
		DONE(arg);
		adv(page);
		plflush();
		return TCL_OK;
	} else if(arg("x0 y0 xopt xtick nxsub yopt ytick nysub",
		          "This functions similarly to plbox() except that the origin of the axes is placed at the user-specified point (x0, y0).")
		      =="axes") {
		PLFLT x0;
		PLFLT y0;
		const char * xopt;
		PLFLT xtick;
		PLINT nxsub;
		const char * yopt;
		PLFLT ytick;
		PLINT nysub;
		arg >> x0 >> y0 >> xopt >> xtick >> nxsub >> yopt >> ytick >> nysub;
		DONE(arg);
		axes(x0, y0, xopt, xtick, nxsub, yopt, ytick, nysub);
		plflush();
		return TCL_OK;
	} else if(arg("nbin x y center",
		          "Plot a histogram using x to store data values and y to store frequencies")
		      =="bin") {
		PLINT nbin;
		TclMatFloat x;
		TclMatFloat y;
		PLINT center;
		arg >> nbin >> x >> y >> center;
		DONE(arg);
		bin(nbin, x, y, center);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Start new page.  Should only be used with pleop().")
		      =="bop") {
		DONE(arg);
		bop();
		plflush();
		return TCL_OK;
	} else if(arg("xopt xtick nxsub yopt ytick nysub",
		          "This draws a box around the current viewport.")
		      =="box") {
		const char * xopt;
		PLFLT xtick;
		PLINT nxsub;
		const char * yopt;
		PLFLT ytick;
		PLINT nysub;
		arg >> xopt >> xtick >> nxsub >> yopt >> ytick >> nysub;
		DONE(arg);
		box(xopt, xtick, nxsub, yopt, ytick, nysub);
		plflush();
		return TCL_OK;
	} else if(arg("xopt xlabel xtick nsubx yopt ylabel ytick nsuby zopt zlabel ztick nsubz",
		          "This is the 3-d analogue of plbox().")
		      =="box3") {
		const char * xopt;
		const char * xlabel;
		PLFLT xtick;
		PLINT nsubx;
		const char * yopt;
		const char * ylabel;
		PLFLT ytick;
		PLINT nsuby;
		const char * zopt;
		const char * zlabel;
		PLFLT ztick;
		PLINT nsubz;
		arg >> xopt >> xlabel >> xtick >> nsubx >> yopt >> ylabel >> ytick >> nsuby >> zopt >> zlabel >> ztick >> nsubz;
		DONE(arg);
		box3(xopt, xlabel, xtick, nsubx, yopt, ylabel, ytick, nsuby, zopt, zlabel, ztick, nsubz);
		plflush();
		return TCL_OK;
	} else if(arg("icol0",
		          "Set color, map 0.  Argument is integer between 0 and 15.").next_arg("int")
		      =="col") {
		PLINT icol0;
		arg >> icol0;
		DONE(arg);
		col0(icol0);
		plflush();
		return TCL_OK;
	} else if(arg("colour1",
		          "Set color, map 1.  Argument is a float between 0. and 1.")
		      =="col") {
		PLFLT colour1;
		arg >> colour1;
		DONE(arg);
		col1(colour1);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Converts input values from relative device coordinates to relative plot coordinates.")
		      =="did2pc") {
		PLFLT xmin;
		PLFLT ymin;
		PLFLT xmax;
		PLFLT ymax;
		DONE(arg);
		did2pc(xmin, ymin, xmax, ymax);
		plflush();
		tcl_ << xmin << " " << ymin << " " << xmax << " " << ymax << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Converts input values from relative plot coordinates to relative device coordinates.")
		      =="dip2dc") {
		PLFLT xmin;
		PLFLT ymin;
		PLFLT xmax;
		PLFLT ymax;
		DONE(arg);
		dip2dc(xmin, ymin, xmax, ymax);
		plflush();
		tcl_ << xmin << " " << ymin << " " << xmax << " " << ymax << " " << result;
		return TCL_OK;
	} else if(arg("xmin xmax ymin ymax just axis",
		          "Simple interface for defining viewport and window.")
		      =="env") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		PLINT just;
		PLINT axis;
		arg >> xmin >> xmax >> ymin >> ymax >> just >> axis;
		DONE(arg);
		env(xmin, xmax, ymin, ymax, just, axis);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "End current page.  Should only be used with plbop().")
		      =="eop") {
		DONE(arg);
		eop();
		plflush();
		return TCL_OK;
	} else if(arg("n xmin xmax y",
		          "Plot horizontal error bars (xmin(i),y(i)) to (xmax(i),y(i))")
		      =="errx") {
		PLINT n;
		TclMatFloat xmin;
		TclMatFloat xmax;
		TclMatFloat y;
		arg >> n >> xmin >> xmax >> y;
		DONE(arg);
		errx(n, xmin, xmax, y);
		plflush();
		return TCL_OK;
	} else if(arg("n x ymin ymax",
		          "Plot vertical error bars (x,ymin(i)) to (x(i),ymax(i))")
		      =="erry") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat ymin;
		TclMatFloat ymax;
		arg >> n >> x >> ymin >> ymax;
		DONE(arg);
		erry(n, x, ymin, ymax);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Advance to the next family file on the next new page")
		      =="famadv") {
		DONE(arg);
		famadv();
		plflush();
		return TCL_OK;
	} else if(arg("n x y",
		          "Pattern fills the polygon bounded by the input points.")
		      =="fill") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		arg >> n >> x >> y;
		DONE(arg);
		fill(n, x, y);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Flushes the output stream.  Use sparingly, if at all.")
		      =="plflush") {
		DONE(arg);
		plflush();
		plflush();
		return TCL_OK;
	} else if(arg("ifont",
		          "Sets the global font flag to 'ifont'.")
		      =="font") {
		PLINT ifont;
		arg >> ifont;
		DONE(arg);
		font(ifont);
		plflush();
		return TCL_OK;
	} else if(arg("fnt",
		          "Load specified font set.")
		      =="fontld") {
		PLINT fnt;
		arg >> fnt;
		DONE(arg);
		fontld(fnt);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Get character default height and current (scaled) height.")
		      =="gchr") {
		PLFLT def;
		PLFLT ht;
		DONE(arg);
		gchr(def, ht);
		plflush();
		tcl_ << def << " " << ht << " " << result;
		return TCL_OK;
	} else if(arg("icol0",
		          "Returns 8 bit RGB values for given color from color map 0.")
		      =="gcol0") {
		PLINT icol0;
		PLINT r;
		PLINT g;
		PLINT b;
		arg >> icol0;
		DONE(arg);
		gcol0(icol0, r, g, b);
		plflush();
		tcl_ << r << " " << g << " " << b << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Retrieve current window into device space.")
		      =="gdidev") {
		PLFLT mar;
		PLFLT aspect;
		PLFLT jx;
		PLFLT jy;
		DONE(arg);
		gdidev(mar, aspect, jx, jy);
		plflush();
		tcl_ << mar << " " << aspect << " " << jx << " " << jy << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Get plot orientation .")
		      =="gdiori") {
		PLFLT rot;
		DONE(arg);
		gdiori(rot);
		plflush();
		tcl_ << rot << " " << result;
		return TCL_OK;
	} else if(arg("setp precision",
		          "Set the floating point precision (in number of places) in numeric labels.")
		      =="prec") {
		PLINT setp;
		PLINT precision;
		arg >> setp >> precision;
		DONE(arg);
		prec(setp, precision);
		plflush();
		return TCL_OK;
	} else if(arg("patt",
		          "Set fill pattern, using one of the predefined patterns.")
		      =="psty") {
		PLINT patt;
		arg >> patt;
		DONE(arg);
		psty(patt);
		plflush();
		return TCL_OK;
	} else if(arg("x y dx dy just text",
		          "Prints out \"text\" at world cooordinate (x,y).")
		      =="ptex") {
		PLFLT x;
		PLFLT y;
		PLFLT dx;
		PLFLT dy;
		PLFLT just;
		const char * text;
		arg >> x >> y >> dx >> dy >> just >> text;
		DONE(arg);
		ptex(x, y, dx, dy, just, text);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Replays contents of plot buffer to current device/file.")
		      =="replot") {
		DONE(arg);
		replot();
		plflush();
		return TCL_OK;
	} else if(arg("r g b",
		          "Set line color by red, green, blue from  0. to 1.")
		      =="rgb") {
		PLFLT r;
		PLFLT g;
		PLFLT b;
		arg >> r >> g >> b;
		DONE(arg);
		rgb(r, g, b);
		plflush();
		return TCL_OK;
	} else if(arg("r g b",
		          "Set line color by 8 bit RGB values.")
		      =="rgb1") {
		PLINT r;
		PLINT g;
		PLINT b;
		arg >> r >> g >> b;
		DONE(arg);
		rgb1(r, g, b);
		plflush();
		return TCL_OK;
	} else if(arg("def scale",
		          "Set character height.")
		      =="schr") {
		PLFLT def;
		PLFLT scale;
		arg >> def >> scale;
		DONE(arg);
		schr(def, scale);
		plflush();
		return TCL_OK;
	} else if(arg("ncol0",
		          "Set number of colors in cmap 0.")
		      =="scmap0n") {
		PLINT ncol0;
		arg >> ncol0;
		DONE(arg);
		scmap0n(ncol0);
		plflush();
		return TCL_OK;
	} else if(arg("ncol1",
		          "Set number of colors in cmap 1.")
		      =="scmap1n") {
		PLINT ncol1;
		arg >> ncol1;
		DONE(arg);
		scmap1n(ncol1);
		plflush();
		return TCL_OK;
	} else if(arg("r g b ncol0",
		          "Set color map 0 colors by 8 bit RGB values.")
		      =="scmap0") {
		TclMatInt r;
		TclMatInt g;
		TclMatInt b;
		PLINT ncol0;
		arg >> r >> g >> b >> ncol0;
		DONE(arg);
		scmap0(r, g, b, ncol0);
		plflush();
		return TCL_OK;
	} else if(arg("r g b ncol1",
		          "Set color map 1 colors by 8 bit RGB values.")
		      =="scmap1") {
		TclMatInt r;
		TclMatInt g;
		TclMatInt b;
		PLINT ncol1;
		arg >> r >> g >> b >> ncol1;
		DONE(arg);
		scmap1(r, g, b, ncol1);
		plflush();
		return TCL_OK;
	} else if(arg("itype npts intensity coord1 coord2 coord3 rev",
		          "Set color map 1 colors using a piece-wise linear relationship between intensity [0,1] (cmap 1 index) and position in HLS or RGB color space.")
		      =="scmap1l") {
		PLINT itype;
		PLINT npts;
		TclMatFloat intensity;
		TclMatFloat coord1;
		TclMatFloat coord2;
		TclMatFloat coord3;
		TclMatInt rev;
		arg >> itype >> npts >> intensity >> coord1 >> coord2 >> coord3 >> rev;
		DONE(arg);
		scmap1l(itype, npts, intensity, coord1, coord2, coord3, rev);
		plflush();
		return TCL_OK;
	} else if(arg("icol0 r g b",
		          "Set a given color from color map 0 by 8 bit RGB value.")
		      =="scol0") {
		PLINT icol0;
		PLINT r;
		PLINT g;
		PLINT b;
		arg >> icol0 >> r >> g >> b;
		DONE(arg);
		scol0(icol0, r, g, b);
		plflush();
		return TCL_OK;
	} else if(arg("r g b",
		          "Set the background color by 8 bit RGB value.")
		      =="scolbg") {
		PLINT r;
		PLINT g;
		PLINT b;
		arg >> r >> g >> b;
		DONE(arg);
		scolbg(r, g, b);
		plflush();
		return TCL_OK;
	} else if(arg("color",
		          "Used to globally turn color output on/off.")
		      =="scolor") {
		PLINT color;
		arg >> color;
		DONE(arg);
		scolor(color);
		plflush();
		return TCL_OK;
	} else if(arg("devnam",
		          "Set the device (keyword) name")
		      =="sdev") {
		const char * devnam;
		arg >> devnam;
		DONE(arg);
		sdev(devnam);
		plflush();
		return TCL_OK;
	} else if(arg("devnam",
		          "Get the device (keyword) name")
		      =="gdev") {
		char * devnam;
		arg >> devnam;
		DONE(arg);
		gdev(devnam);
		plflush();
		return TCL_OK;
	} else if(arg("mar aspect jx jy",
		          "Set window into device space using margin, aspect ratio, and justification.")
		      =="sdidev") {
		PLFLT mar;
		PLFLT aspect;
		PLFLT jx;
		PLFLT jy;
		arg >> mar >> aspect >> jx >> jy;
		DONE(arg);
		sdidev(mar, aspect, jx, jy);
		plflush();
		return TCL_OK;
	} else if(arg("dimxmin dimxmax dimymin dimymax dimxpmm dimypmm",
		          "Set up transformation from metafile coordinates.")
		      =="sdimap") {
		PLINT dimxmin;
		PLINT dimxmax;
		PLINT dimymin;
		PLINT dimymax;
		PLFLT dimxpmm;
		PLFLT dimypmm;
		arg >> dimxmin >> dimxmax >> dimymin >> dimymax >> dimxpmm >> dimypmm;
		DONE(arg);
		sdimap(dimxmin, dimxmax, dimymin, dimymax, dimxpmm, dimypmm);
		plflush();
		return TCL_OK;
	} else if(arg("rot",
		          "Set plot orientation, specifying rotation in units of pi/2.")
		      =="sdiori") {
		PLFLT rot;
		arg >> rot;
		DONE(arg);
		sdiori(rot);
		plflush();
		return TCL_OK;
	} else if(arg("xmin ymin xmax ymax",
		          "Set window into plot space.")
		      =="sdiplt") {
		PLFLT xmin;
		PLFLT ymin;
		PLFLT xmax;
		PLFLT ymax;
		arg >> xmin >> ymin >> xmax >> ymax;
		DONE(arg);
		sdiplt(xmin, ymin, xmax, ymax);
		plflush();
		return TCL_OK;
	} else if(arg("xmin ymin xmax ymax",
		          "Set window into plot space incrementally (zoom).")
		      =="sdiplz") {
		PLFLT xmin;
		PLFLT ymin;
		PLFLT xmax;
		PLFLT ymax;
		arg >> xmin >> ymin >> xmax >> ymax;
		DONE(arg);
		sdiplz(xmin, ymin, xmax, ymax);
		plflush();
		return TCL_OK;
	} else if(arg("esc",
		          "Set the escape character for text strings.")
		      =="sesc") {
		char esc;
		arg >> esc;
		DONE(arg);
		sesc(esc);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Get the escape character for text strings.")
		      =="gesc") {
		char esc;
		DONE(arg);
		gesc(esc);
		plflush();
		tcl_ << esc << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Retrieve current window into plot space.")
		      =="gdiplt") {
		PLFLT xmin;
		PLFLT ymin;
		PLFLT xmax;
		PLFLT ymax;
		DONE(arg);
		gdiplt(xmin, ymin, xmax, ymax);
		plflush();
		tcl_ << xmin << " " << ymin << " " << xmax << " " << ymax << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Get family file parameters.")
		      =="gfam") {
		PLINT fam;
		PLINT num;
		PLINT bmax;
		DONE(arg);
		gfam(fam, num, bmax);
		plflush();
		tcl_ << fam << " " << num << " " << bmax << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Get output device parameters.")
		      =="gpage") {
		PLFLT xp;
		PLFLT yp;
		PLINT xleng;
		PLINT yleng;
		PLINT xoff;
		PLINT yoff;
		DONE(arg);
		gpage(xp, yp, xleng, yleng, xoff, yoff);
		plflush();
		tcl_ << xp << " " << yp << " " << xleng << " " << yleng << " " << xoff << " " << yoff << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Switches to graphics screen.")
		      =="gra") {
		DONE(arg);
		gra();
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Get subpage boundaries in absolute coordinates.")
		      =="gspa") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		DONE(arg);
		gspa(xmin, xmax, ymin, ymax);
		plflush();
		tcl_ << xmin << " " << xmax << " " << ymin << " " << ymax << " " << result;
		return TCL_OK;
	} else if(arg("ver",
		          "Get the current library version number.")
		      =="gver") {
		char * ver;
		arg >> ver;
		DONE(arg);
		gver(ver);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Get x axis labeling parameters.")
		      =="gxax") {
		PLINT digmax;
		PLINT digits;
		DONE(arg);
		gxax(digmax, digits);
		plflush();
		tcl_ << digmax << " " << digits << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Get y axis labeling parameters.")
		      =="gyax") {
		PLINT digmax;
		PLINT digits;
		DONE(arg);
		gyax(digmax, digits);
		plflush();
		tcl_ << digmax << " " << digits << " " << result;
		return TCL_OK;
	} else if(arg("",
		          "Get z axis labeling parameters.")
		      =="gzax") {
		PLINT digmax;
		PLINT digits;
		DONE(arg);
		gzax(digmax, digits);
		plflush();
		tcl_ << digmax << " " << digits << " " << result;
		return TCL_OK;
	} else if(arg("n data datmin datmax nbin oldwin",
		          "Draws a histogram of n values of a variable in array data[0..n-1]. pltclgen not able to handle Tcl Matricies yet.")
		      =="hist") {
		PLINT n;
		TclMatFloat data;
		PLFLT datmin;
		PLFLT datmax;
		PLINT nbin;
		PLINT oldwin;
		arg >> n >> data >> datmin >> datmax >> nbin >> oldwin;
		DONE(arg);
		hist(n, data, datmin, datmax, nbin, oldwin);
		plflush();
		return TCL_OK;
	} else if(arg("h l s",
		          "Set current color (map 0) by hue, lightness, and saturation.")
		      =="hls") {
		PLFLT h;
		PLFLT l;
		PLFLT s;
		arg >> h >> l >> s;
		DONE(arg);
		hls(h, l, s);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Initializes PLplot, using preset or default options.")
		      =="init") {
		DONE(arg);
		init();
		plflush();
		return TCL_OK;
	} else if(arg("x1 y1 x2 y2",
		          "Draws a line segment from (x1, y1) to (x2, y2).")
		      =="join") {
		PLFLT x1;
		PLFLT y1;
		PLFLT x2;
		PLFLT y2;
		arg >> x1 >> y1 >> x2 >> y2;
		DONE(arg);
		join(x1, y1, x2, y2);
		plflush();
		return TCL_OK;
	} else if(arg("xlabel ylabel tlabel",
		          "Simple routine for labelling graphs.")
		      =="lab") {
		const char * xlabel;
		const char * ylabel;
		const char * tlabel;
		arg >> xlabel >> ylabel >> tlabel;
		DONE(arg);
		lab(xlabel, ylabel, tlabel);
		plflush();
		return TCL_OK;
	} else if(arg("x y z",
		          "sets the position of the light source")
		      =="lightsource") {
		PLFLT x;
		PLFLT y;
		PLFLT z;
		arg >> x >> y >> z;
		DONE(arg);
		lightsource(x, y, z);
		plflush();
		return TCL_OK;
	} else if(arg("n x y",
		          "Draws line segments connecting a series of points.")
		      =="line") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		arg >> n >> x >> y;
		DONE(arg);
		line(n, x, y);
		plflush();
		return TCL_OK;
	} else if(arg("n x y z",
		          "Draws a line in 3 space. pltclgen...")
		      =="line3") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		TclMatFloat z;
		arg >> n >> x >> y >> z;
		DONE(arg);
		line3(n, x, y, z);
		plflush();
		return TCL_OK;
	} else if(arg("lin",
		          "Set line style.")
		      =="lsty") {
		PLINT lin;
		arg >> lin;
		DONE(arg);
		lsty(lin);
		plflush();
		return TCL_OK;
	} else if(arg("side disp pos just text",
		          "Prints out \"text\" at specified position relative to viewport.")
		      =="mtex") {
		const char * side;
		PLFLT disp;
		PLFLT pos;
		PLFLT just;
		const char * text;
		arg >> side >> disp >> pos >> just >> text;
		DONE(arg);
		mtex(side, disp, pos, just, text);
		plflush();
		return TCL_OK;
	} else if(arg("nlin inc del",
		          "Set fill pattern directly.")
		      =="pat") {
		PLINT nlin;
		TclMatInt inc;
		TclMatInt del;
		arg >> nlin >> inc >> del;
		DONE(arg);
		pat(nlin, inc, del);
		plflush();
		return TCL_OK;
	} else if(arg("n x y code",
		          "Plots array y against x for n points using ASCII code \"code\".")
		      =="poin") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		PLINT code;
		arg >> n >> x >> y >> code;
		DONE(arg);
		poin(n, x, y, code);
		plflush();
		return TCL_OK;
	} else if(arg("n x y z code",
		          "Draws a series of points in 3 space.")
		      =="poin3") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		TclMatFloat z;
		PLINT code;
		arg >> n >> x >> y >> z >> code;
		DONE(arg);
		poin3(n, x, y, z, code);
		plflush();
		return TCL_OK;
	} else if(arg("n x y z draw",
		          "Draws a polygon in 3 space.")
		      =="poly3") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		TclMatFloat z;
		TclMatInt draw;
		arg >> n >> x >> y >> z >> draw;
		DONE(arg);
		poly3(n, x, y, z, draw);
		plflush();
		return TCL_OK;
	} else if(arg("fam num bmax",
		          "Set family file parameters")
		      =="sfam") {
		PLINT fam;
		PLINT num;
		PLINT bmax;
		arg >> fam >> num >> bmax;
		DONE(arg);
		sfam(fam, num, bmax);
		plflush();
		return TCL_OK;
	} else if(arg("fnam",
		          "Set the output file name.")
		      =="sfnam") {
		const char * fnam;
		arg >> fnam;
		DONE(arg);
		sfnam(fnam);
		plflush();
		return TCL_OK;
	} else if(arg("fnam",
		          "Get the output file name.")
		      =="gfnam") {
		char * fnam;
		arg >> fnam;
		DONE(arg);
		gfnam(fnam);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Get the current run level.")
		      =="glevel") {
		PLINT level;
		DONE(arg);
		glevel(level);
		plflush();
		tcl_ << level << " " << result;
		return TCL_OK;
	} else if(arg("def scale",
		          "Set up lengths of major tick marks.")
		      =="smaj") {
		PLFLT def;
		PLFLT scale;
		arg >> def >> scale;
		DONE(arg);
		smaj(def, scale);
		plflush();
		return TCL_OK;
	} else if(arg("def scale",
		          "Set up lengths of minor tick marks.")
		      =="smin") {
		PLFLT def;
		PLFLT scale;
		arg >> def >> scale;
		DONE(arg);
		smin(def, scale);
		plflush();
		return TCL_OK;
	} else if(arg("ori",
		          "Set orientation.  Must be done before calling plinit.")
		      =="sori") {
		PLINT ori;
		arg >> ori;
		DONE(arg);
		sori(ori);
		plflush();
		return TCL_OK;
	} else if(arg("xp yp xleng yleng xoff yoff",
		          "Set output device parameters.  Usually ignored by the driver.")
		      =="spage") {
		PLFLT xp;
		PLFLT yp;
		PLINT xleng;
		PLINT yleng;
		PLINT xoff;
		PLINT yoff;
		arg >> xp >> yp >> xleng >> yleng >> xoff >> yoff;
		DONE(arg);
		spage(xp, yp, xleng, yleng, xoff, yoff);
		plflush();
		return TCL_OK;
	} else if(arg("paus",
		          "Set the pause (on end-of-page) status.")
		      =="spause") {
		PLINT paus;
		arg >> paus;
		DONE(arg);
		spause(paus);
		plflush();
		return TCL_OK;
	} else if(arg("strm",
		          "Set stream number.")
		      =="sstrm") {
		PLINT strm;
		arg >> strm;
		DONE(arg);
		sstrm(strm);
		plflush();
		return TCL_OK;
	} else if(arg("nx ny",
		          "Set the number of subwindows in x and y.")
		      =="ssub") {
		PLINT nx;
		PLINT ny;
		arg >> nx >> ny;
		DONE(arg);
		ssub(nx, ny);
		plflush();
		return TCL_OK;
	} else if(arg("def scale",
		          "Set symbol height.")
		      =="ssym") {
		PLFLT def;
		PLFLT scale;
		arg >> def >> scale;
		DONE(arg);
		ssym(def, scale);
		plflush();
		return TCL_OK;
	} else if(arg("nms mark space",
		          "Set up a new line style.")
		      =="styl") {
		PLINT nms;
		TclMatInt mark;
		TclMatInt space;
		arg >> nms >> mark >> space;
		DONE(arg);
		styl(nms, mark, space);
		plflush();
		return TCL_OK;
	} else if(arg("xmin xmax ymin ymax",
		          "Sets the edges of the viewport to the specified absolute coordinates.")
		      =="svpa") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		arg >> xmin >> xmax >> ymin >> ymax;
		DONE(arg);
		svpa(xmin, xmax, ymin, ymax);
		plflush();
		return TCL_OK;
	} else if(arg("digmax digits",
		          "Set x axis labeling parameters.")
		      =="sxax") {
		PLINT digmax;
		PLINT digits;
		arg >> digmax >> digits;
		DONE(arg);
		sxax(digmax, digits);
		plflush();
		return TCL_OK;
	} else if(arg("digmax ?digits?",
		          "Set y axis labeling parameters.")
		      =="syax") {
		PLINT digmax;
		PLINT	 digits =  0;
		arg >> digmax >> optional >> digits;
		DONE(arg);
		syax(digmax, digits);
		plflush();
		return TCL_OK;
	} else if(arg("n x y code",
		          "Plots array y against x for n points using Hershey symbol \"code\".")
		      =="sym") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		PLINT code;
		arg >> n >> x >> y >> code;
		DONE(arg);
		sym(n, x, y, code);
		plflush();
		return TCL_OK;
	} else if(arg("digmax digits",
		          "Set z axis labeling parameters.")
		      =="szax") {
		PLINT digmax;
		PLINT digits;
		arg >> digmax >> digits;
		DONE(arg);
		szax(digmax, digits);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Switches to text screen.")
		      =="text") {
		DONE(arg);
		text();
		plflush();
		return TCL_OK;
	} else if(arg("aspect",
		          "Sets the edges of the viewport with the given aspect ratio, leaving room for labels.")
		      =="vasp") {
		PLFLT aspect;
		arg >> aspect;
		DONE(arg);
		vasp(aspect);
		plflush();
		return TCL_OK;
	} else if(arg("xmin xmax ymin ymax aspect",
		          "Creates the largest viewport of the specified aspect ratio that fits within the specified normalized subpage coordinates.")
		      =="vpas") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		PLFLT aspect;
		arg >> xmin >> xmax >> ymin >> ymax >> aspect;
		DONE(arg);
		vpas(xmin, xmax, ymin, ymax, aspect);
		plflush();
		return TCL_OK;
	} else if(arg("xmin xmax ymin ymax",
		          "Creates a viewport with the specified normalized subpage coordinates.")
		      =="vpor") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		arg >> xmin >> xmax >> ymin >> ymax;
		DONE(arg);
		vpor(xmin, xmax, ymin, ymax);
		plflush();
		return TCL_OK;
	} else if(arg("",
		          "Defines a \"standard\" viewport with seven character heights for the left margin and four character heights everywhere else.")
		      =="vsta") {
		DONE(arg);
		vsta();
		plflush();
		return TCL_OK;
	} else if(arg("basex basey height xmin0 xmax0 ymin0 ymax0 zmin0 zmax0 alt az",
		          "Set up a window for three-dimensional plotting.")
		      =="w3d") {
		PLFLT basex;
		PLFLT basey;
		PLFLT height;
		PLFLT xmin0;
		PLFLT xmax0;
		PLFLT ymin0;
		PLFLT ymax0;
		PLFLT zmin0;
		PLFLT zmax0;
		PLFLT alt;
		PLFLT az;
		arg >> basex >> basey >> height >> xmin0 >> xmax0 >> ymin0 >> ymax0 >> zmin0 >> zmax0 >> alt >> az;
		DONE(arg);
		w3d(basex, basey, height, xmin0, xmax0, ymin0, ymax0, zmin0, zmax0, alt, az);
		plflush();
		return TCL_OK;
	} else if(arg("width",
		          "Set pen width.")
		      =="wid") {
		PLINT width;
		arg >> width;
		DONE(arg);
		wid(width);
		plflush();
		return TCL_OK;
	} else if(arg("xmin xmax ymin ymax",
		          "Set up world coordinates of the viewport boundaries (2d plots).")
		      =="wind") {
		PLFLT xmin;
		PLFLT xmax;
		PLFLT ymin;
		PLFLT ymax;
		arg >> xmin >> xmax >> ymin >> ymax;
		DONE(arg);
		wind(xmin, xmax, ymin, ymax);
		plflush();
		return TCL_OK;
	} else 
#endif
	if(arg("n x y",
	"Plots array y against x for n points using single points")
		      =="dots") {
		PLINT n;
		TclMatFloat x;
		TclMatFloat y;
		arg >> n >> x >> y;
		DONE(arg);
		dots(n,x,y);
		plflush();
		return TCL_OK;
	} else if(arg("?x y? z opt side",
           "Plots a 3-d representation of the function z[x][y].")
       =="plot3d") {
		PLINT nx, ny, opt, side;
		PLFLT *x=0, *y=0, **z=0;
		TclMatFloat matx, maty;
		TclMatFloat2 matz;
		switch(arg) {
		  case 5:
			arg >> matx >> maty >> matz >> opt >> side;
			DONE(arg);
			nx = matx.dim_size(0);
			ny = matx.dim_size(0);
			
			if (matz.dim_size(0) != nx	|| matz.dim_size(1) != ny	) {
				tcl_ << "Inconsistent dimensions" << tcl_error;
				return TCL_ERROR;
			}
			x =	matx;
			y =	maty;
	
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			plot3d(	x, y, z, nx, ny, opt, side );
			free_mem(z);
			break;
		  case 3:
		  default:  
			arg >> matz >> opt >> side;
			DONE(arg);
			nx = matz.dim_size(0); ny	= matz.dim_size(1);
			x = (PLFLT*) malloc(nx * sizeof(PLFLT));
			y = (PLFLT*) malloc(ny * sizeof(PLFLT));
			for(short i=0;i<nx;i++)
				x[i] = (PLFLT) i;
			for(short i=0;i<ny;i++)
				y[i] = (PLFLT) i;
	
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			plot3d(	x, y, z, nx, ny, opt, side );
			free_mem(x);
			free_mem(y);
			free_mem(z);
			break;
		}
	
		plflush();
		return TCL_OK;
    } else if(arg("?x y? z side",
                  "Plots a surface shaded 3-d representation of the function z[x][y].")
              =="plotsh3d") {		
		PLINT nx, ny, side;
		PLFLT *x=0, *y=0, **z=0;
		TclMatFloat matx, maty;
		TclMatFloat2 matz;
		switch(arg) {
		  case 4:
			arg >> matx >> maty >> matz >> side;
			DONE(arg);
			nx = matx.dim_size(0);
			ny = matx.dim_size(0);
			
			if (matz.dim_size(0) != nx	|| matz.dim_size(1) != ny	) {
				tcl_ << "Inconsistent dimensions" << tcl_error;
				return TCL_ERROR;
			}
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			plotsh3d(matx, maty, z, nx, ny, side );
			free_mem(z);
			break;
		  case 2:
		  default:
			arg >> matz >> side;
			DONE(arg);
			nx = matz.dim_size(0); ny	= matz.dim_size(1);
			x = (PLFLT*) malloc(nx * sizeof(PLFLT));
			y = (PLFLT*) malloc(ny * sizeof(PLFLT));
			for(short i=0;i<nx;i++)
				x[i] = (PLFLT) i;
			for(short i=0;i<ny;i++)
				y[i] = (PLFLT) i;
	
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			plotsh3d(x, y, z, nx, ny, side );
			free_mem(x);
			free_mem(y);
			free_mem(z);
			break;
		}
	
		plflush();
		return TCL_OK;
	} else if(arg("option argument",
	              "Process input strings, treating them as an option and argument pair.")
	          =="setopt"){	
		const char * a1, *a2;
		arg >> a1 >> a2;
		DONE(arg);
		SetOpt((char*) a1, (char*) a2);
	
		plflush();
		return tcl_;
	} else if(arg("type minlong maxlong minlat maxlat",
	              "Plot continental outline in world coordinates.")
	         =="map"){	
		const char * a1;
		PLFLT minlong, maxlong, minlat, maxlat;
		arg >> a1 >> minlong >> maxlong >> minlat >> maxlat;
		DONE(arg);
		map(NULL,(char*)a1,minlong,maxlong,minlat,maxlat);
		
		plflush();
		return tcl_;
	} else if(arg("dlong dlat minlong maxlong minlat maxlat",
	              "Plot the latitudes and longitudes on the background.")
	         =="meridians"){	
		PLFLT dlong, dlat, minlong, maxlong, minlat, maxlat;
 		arg >> dlong >> dlat >> minlong >> maxlong >> minlat >> maxlat;
		DONE(arg);
		meridians(NULL,dlong,dlat,minlong,maxlong,minlat,maxlat);
		
		plflush();
		return tcl_;
	} else if(arg("z xmin xmax	ymin ymax sh_min sh_max sh_cmap sh_color sh_width	min_col	min_wid	max_col	max_wid	rect [pltr x y]	[wrap]",
	              "Shade region.")
	          =="shade"){	
		tclMatrix *matPtr;
		TclMatFloat2 mattrx, mattry;
		TclMatFloat2 matz;
		PLFLT xmin,	xmax, ymin,	ymax, sh_min, sh_max, sh_col;

		PLINT sh_cmap =1, sh_wid =2;
		PLINT min_col =1, min_wid =0, max_col =0, max_wid =0;
		PLINT rect =1;
		char *pltrname = "pltr0";
		void (*pltr) (PLFLT, PLFLT,	PLFLT *, PLFLT *, PLPointer);
		PLPointer pltr_data	= NULL;
		PLcGrid	 cgrid1;
		PLcGrid2 cgrid2;
		PLINT pwrap = 0;
		int	nx,	ny;
		arg >> matz >> xmin >> xmax >> ymin >> ymax 
		    >> sh_min >> sh_max >> sh_cmap >> sh_col;
		    
		if(arg.args_left()) {
			arg >> sh_wid >> min_col >> min_wid >> max_col >> max_wid >> rect;
			if(arg.args_left()) {
				arg >> pltrname >> mattrx >> mattry
					>> optional >> pwrap;
			}
		}
		DONE(arg);
	
		nx = matz.dim_size(0);
		ny = matz.dim_size(1);
	
		tclmateval_modx	= nx;
		tclmateval_mody	= ny;
	
	/* Figure out which	coordinate transformation model	is being used, and setup
	   accordingly.	*/
	
		if ( !strcmp( pltrname,	"pltr0"	) )	{
			pltr = pltr0;
	
		/* wrapping	is only	supported for pltr2. */
			if (pwrap) {
				tcl_ << "Must use pltr2 if	want wrapping." << tcl_error;
				return TCL_ERROR;
			}
		} else if	( !strcmp( pltrname, "pltr1" ) ) {
			pltr = pltr1;
			cgrid1.xg =	mattrx;
			cgrid1.nx =	nx;
			cgrid1.yg =	mattry;
			cgrid1.ny =	ny;
	
		/* wrapping	is only	supported for pltr2. */
			if (pwrap) {
				tcl_ << "Must use pltr2 if	want wrapping." << tcl_error;
				return TCL_ERROR;
			}
	
			pltr_data =	&cgrid1;
		} else if	( !strcmp( pltrname, "pltr2" ) ) {
		/* printf( "plshade, setting up	for	pltr2\n" );	*/
			if (!pwrap) {
			/* printf( "plshade, no	wrapping is	needed.\n" ); */
				Alloc2dGrid( &cgrid2.xg, nx, ny );
				Alloc2dGrid( &cgrid2.yg, nx, ny );
				cgrid2.nx =	nx;
				cgrid2.ny =	ny;
	
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
			} else if	(pwrap == 1)	{
				Alloc2dGrid( &cgrid2.xg, nx+1, ny	);
				Alloc2dGrid( &cgrid2.yg, nx+1, ny	);
				cgrid2.nx =	nx+1;
				cgrid2.ny =	ny;
	
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
	
				for(short j=0; j	< ny; j++ )	{
					cgrid2.xg[nx][j] = cgrid2.xg[0][j];
					cgrid2.yg[nx][j] = cgrid2.yg[0][j];
				}
	
				nx++;
			} else if	(pwrap == 2)	{
				Alloc2dGrid( &cgrid2.xg, nx, ny+1	);
				Alloc2dGrid( &cgrid2.yg, nx, ny+1	);
				cgrid2.nx =	nx;
				cgrid2.ny =	ny+1;
	
				matPtr = mattrx;
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				matPtr = mattry;
				for(short i=0; i	< nx; i++ )
					for(short j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
	
				for(short i=0; i	< nx; i++ )	{
					cgrid2.xg[i][ny] = cgrid2.xg[i][0];
					cgrid2.yg[i][ny] = cgrid2.yg[i][0];
				}
	
				ny++;
			} else {
				tcl_ << "Invalid wrap specifier, must be <empty>, 1	or 2." << tcl_error;
				return TCL_ERROR;
			}
	
			pltr = pltr2;
			pltr_data =	&cgrid2;
		} else {
			tcl_ << "Unrecognized coordinate transformation spec:"
			     << pltrname << "must be pltr0 pltr1 or pltr2." << tcl_error;
			return TCL_ERROR;
		}
	
	/* Now go make the plot. */
	
		fshade( tclMatrix_feval, (tclMatrix *) matz, NULL, NULL, nx, ny,
				  xmin,	xmax, ymin,	ymax,
				  sh_min, sh_max, sh_cmap, sh_col, sh_wid,
				  min_col, min_wid,	max_col, max_wid,
				  plfill, rect,	pltr, pltr_data	);
	
	/* Now free	up any space which got allocated for our coordinate	trickery. */
	
		if (pltr ==	pltr1) {
		/* Hmm,	actually, nothing to do	here currently,	since we just used the
		   Tcl Matrix data directly, rather	than allocating	private	space. */
		} else if	(pltr == pltr2)	{
		/* printf( "plshade, freeing space for grids used in pltr2\n" ); */
			Free2dGrid( cgrid2.xg, nx, ny	);
			Free2dGrid( cgrid2.yg, nx, ny	);
		}
	
		plflush();
		return tcl_;
	} else if(arg("opt z x y",
	              "Plots a mesh representation of the function z[x][y].")
	          =="mesh"){
		PLINT nx, ny, opt;
		PLFLT *x, *y, **z;
		TclMatFloat matx, maty;
		TclMatFloat2 matz;
		arg >> opt >> matz;
		if(arg.args_left()) {
			arg >> matx >> maty;
			DONE(arg);
			nx = matx.dim_size(0); ny	= maty.dim_size(0);
			x =	matx;
			y =	maty;
	
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			mesh(x, y, z, nx, ny, opt );
			free_mem(z);
		} else {
			DONE(arg);
			nx = matz.dim_size(0); ny	= matz.dim_size(1);
			x = (PLFLT*) malloc(nx * sizeof(PLFLT));
			y = (PLFLT*) malloc(ny * sizeof(PLFLT));
			for(short i=0;i<nx;i++)
				x[i] = (PLFLT) i;
			for(short i=0;i<ny;i++)
				y[i] = (PLFLT) i;	
			z =	(PLFLT **) malloc( nx *	sizeof(PLFLT *)	);
			for(short i=0; i	< nx; i++ )
				z[i] = &matz(i,0);
			plmesh(	x, y, z, nx, ny, opt );
			free_mem(x);
			free_mem(y);
			free_mem(z);
		}	
		plflush();
		return tcl_;
	} else if(arg("",
	              "Draws a contour plot from data in f(nx,ny).")
	          =="cont"){	
		tclMatrix *matPtr;
		PLINT nx, ny, kx, lx, ky, ly, nclev;
		char *pltrname = "pltr0";
		TclMatFloat mattrx, mattry;
		TclMatFloat2 matf, matclev;
		
		int	arg3_is_kx = 1,	i, j;
		void (*pltr) (PLFLT, PLFLT,	PLFLT *, PLFLT *, PLPointer);
		PLPointer pltr_data	= NULL;
		PLcGrid	 cgrid1;
		PLcGrid2 cgrid2;
	
		int	pwrap = 0;
	
		arg >> matf;
	
	/* Now check the next argument.	 If	it is all digits, then it must be kx,
	   otherwise it	is the name	of clev. */
		if(!arg.peek_is_int()) 
			arg3_is_kx = 0;
	
		if (arg3_is_kx)	{
		/* Peel	off	the	ones we	need */
			arg >> kx >> lx >> ky >> ly;
	
		}
	
		arg >> matclev;
	
	/* Now handle trailing optional	parameters,	if any */
	
		if (!arg.empty()) {
		/* There is	a pltr spec, parse it. */
			arg >> pltrname >> mattrx >> mattry;
			arg >> optional >> pwrap;
		}
		DONE(arg);
		nx = matf.dim_size(0);
		ny = matf.dim_size(1);

		tclmateval_modx	= nx;
		tclmateval_mody	= ny;

		if (!arg3_is_kx) {
			kx = 1;	lx = nx;
			ky = 1;	ly = ny;
		}
		
		nclev =	matclev.dim_size(0);
		
	/* Now we need to set up the data for contouring. */
	
		if ( !strcmp( pltrname,	"pltr0"	) )	{
			pltr = pltr0;
	
		/* wrapping	is only	supported for pltr2. */
			if (pwrap) {
				tcl_ << "Must use pltr2 if	want wrapping." << tcl_error;
				return TCL_ERROR;
			}
		} else if	( !strcmp( pltrname, "pltr1" ) ) {
			pltr = pltr1;
			cgrid1.xg =	mattrx;
			cgrid1.nx =	nx;
			cgrid1.yg =	mattry;
			cgrid1.ny =	ny;
	
		/* wrapping	is only	supported for pltr2. */
			if (pwrap) {
				tcl_ << "Must use pltr2 if	want wrapping." << tcl_error;
				return TCL_ERROR;
			}
	
			pltr_data =	&cgrid1;
		} else if	( !strcmp( pltrname, "pltr2" ) ) {
		/* printf( "plcont,	setting	up for pltr2\n"	); */
			if (!pwrap) {
			/* printf( "plcont,	no wrapping	is needed.\n" ); */
				Alloc2dGrid( &cgrid2.xg, nx, ny );
				Alloc2dGrid( &cgrid2.yg, nx, ny );
				cgrid2.nx =	nx;
				cgrid2.ny =	ny;
	
				matPtr = mattrx;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				matPtr = mattry;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
			} else if	(pwrap == 1)	{
				Alloc2dGrid( &cgrid2.xg, nx+1, ny	);
				Alloc2dGrid( &cgrid2.yg, nx+1, ny	);
				cgrid2.nx =	nx+1;
				cgrid2.ny =	ny;
	
				matPtr = mattrx;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				matPtr = mattry;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
	
				for( j=0; j	< ny; j++ )	{
					cgrid2.xg[nx][j] = cgrid2.xg[0][j];
					cgrid2.yg[nx][j] = cgrid2.yg[0][j];
				}
	
				nx++;
			} else if	(pwrap == 2)	{
				Alloc2dGrid( &cgrid2.xg, nx, ny+1	);
				Alloc2dGrid( &cgrid2.yg, nx, ny+1	);
				cgrid2.nx =	nx;
				cgrid2.ny =	ny+1;
	
				matPtr = mattrx;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.xg[i][j]	= mattrx(i,j);
	
				matPtr = mattry;
				for( i=0; i	< nx; i++ )
					for( j=0; j	< ny; j++ )
						cgrid2.yg[i][j]	= mattry(i,j);
	
				for( i=0; i	< nx; i++ )	{
					cgrid2.xg[i][ny] = cgrid2.xg[i][0];
					cgrid2.yg[i][ny] = cgrid2.yg[i][0];
				}
	
				ny++;
			} else {
				tcl_ << "Invalid wrap specifier, must be <empty>, 1	or 2." << tcl_error;
				return TCL_ERROR;
			}
	
			pltr = pltr2;
			pltr_data =	&cgrid2;
		} else {
			tcl_ << "Unrecognized	coordinate transformation spec:"
				 << pltrname <<	", must	be pltr0 pltr1 or pltr2." << tcl_error;
			return TCL_ERROR;
		}
	/*
		printf(	"plcont: nx=%d ny=%d kx=%d lx=%d ky=%d ly=%d\n",
				nx,	ny,	kx,	lx,	ky,	ly );
		printf(	"plcont: tclmateval_modx=%d	tclmateval_mody=%d\n",
				tclmateval_modx, tclmateval_mody );
		printf(	"plcont: nclev=%d\n", nclev	);
		*/
	
	/* contour the data.*/
	
		fcont(tclMatrix_feval, (tclMatrix *) matf,	nx,	ny,
			  kx, lx, ky, ly, matclev, nclev, pltr, pltr_data );
	
	/* Now free	up any space which got allocated for our coordinate	trickery. */
	
		if (pltr ==	pltr1) {
		/* Hmm,	actually, nothing to do	here currently,	since we just used the
		   Tcl Matrix data directly, rather	than allocating	private	space. */
		} else if	(pltr == pltr2)	{
		/* printf( "plcont,	freeing	space for grids	used in	pltr2\n" );	*/
			Free2dGrid( cgrid2.xg, nx, ny	);
			Free2dGrid( cgrid2.yg, nx, ny	);
		}
	
		plflush();
		return tcl_;
	} else {
		return tcl_object::parse_tcl_command(arg);
	}
}



