
/***********************************************************

    File:       tclstdio.h

    Contents:   C stdio redirection to Tcl procedures

    Author:     Dr. Rico Cozzio,

                COMEXAR Engineering AG
                Breitenstr. 15
                Postfach 441
                CH-8853 Lachen
                Switzerland

    E-mail:     cozzio@ln.active.ch
    WWW:        http://www2.active.ch/~cozzio/index.html

    History:    22.12.1995  First version
                10.02.1996  Separate functions for writing
                            to stdout and stderr
                18.07.1996  Cleaned up for distribution

    Usage:      Include this file instead of stdio.h in
                every C file that uses stdio and define
                the symbol TCLSTDIO.
                Additionally, the following Tcl procedures
                must be defined:
                    stdio_puts_stdout   string
                    stdio_puts_stderr   string
                    stdio_read          bytes

**************** LEGAL STUFF ********************************

    Copyright (c) COMEXAR Engineering AG, 1996

    License is granted to copy, to use, and to make and to use
    derivative works for any purpose, provided that this
    copyright notice and this license notice is included in all
    copies and any derivatives works.
    The licensee acknowleges that Rico Cozzio and COMEXAR
    Corporation have no liability for licensee's use
    or for any derivative works  made by licensee.
    The COMEXAR name shall not be used in any advertising or the
    like without its written permission.

    This software is provided AS IS. COMEXAR CORPORATION
    DISCLAIMS AND LICENSEE AGREES THAT ALL WARRANTIES, EXPRESS
    OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE. NOTWITHSTANDING ANY OTHER PROVISION CONTAINED
    HEREIN, ANY LIABILITY FOR DAMAGES RESULTING FROM THE
    SOFTWARE OR ITS USE IS EXPRESSLY DISCLAIMED, INCLUDING
    CONSEQUENTIAL OR ANY OTHER INDIRECT DAMAGES, WHETHER ARISING
    IN CONTRACT, TORT (INCLUDING NEGLIGENCE) OR STRICT LIABILITY,
    EVEN IF COMEXAR CORPORATION IS ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGES."

    This code was adapted from the TkNT distribution by
    Gordon Chaffee, see acknowledgement below.
    However, almost none of his original code survived.

*************************************************************

    winstdio.h

    Copyright (c) 1994 Software Research Associates, Inc.

    Permission to use, copy, modify, and distribute this software and its
    documentation for any purpose and without fee is hereby granted, provided
    that the above copyright notice appear in all copies and that both that
    copyright notice and this permission notice appear in supporting
    documentation, and that the name of Software Research Associates not be
    used in advertising or publicity pertaining to distribution of the
    software without specific, written prior permission.  Software Research
    Associates makes no representations about the suitability of this software
    for any purpose.  It is provided "as is" without express or implied
    warranty.

*************** END OF LEGAL STUFF *************************/


#ifndef _TCLSTDIO_H
#define _TCLSTDIO_H

#include <stdio.h>

#ifdef TCLSTDIO

#ifdef __cplusplus
extern "C" {
#endif

/*
        Redirect stdio functions for stdin, stdout & stderr
        to tcl procedures
*/

#ifdef REDEFINESTDIO

/*
        Redefine stdin, stdout & stderr macros
*/

#undef stdin
#undef stdout
#undef stderr

#define stdin  (FILE *)0
#define stdout (FILE *)1
#define stderr (FILE *)2

#endif

/*
        Redefine standard functions
*/

#define printf          tclio_printf
#define fgetc(f)                tclio_fgetc(f)
#ifdef getc
#undef getc
#endif
#define getc(f)         tclio_fgetc(f)
#define ungetc(c,f)             tclio_fungetc(c,f)
#define fputs(s, f)             tclio_fputs(s, f)
#ifdef putc
#undef putc
#endif
#define putc(c, f)              tclio_fputc(c, f)
#define fputc(c, f)             tclio_fputc(c, f)
#define fprintf         tclio_fprintf
#define fgets(b, n, f)  tclio_fgets(b, n, f)
#define clearerr(f)             tclio_clearerr(f)
#define fileno(f)               tclio_fileno(f)
#define fflush(f)               tclio_fflush(f)
#undef ferror
#undef feof
#define ferror(f)               tclio_ferror(f)
#define feof(f)         tclio_feof(f)
/*#define tmpnam(f)		tclio_tmpnam(f)*/
/* #define tmpfile			tclio_tmpfile */
	
extern int      tclio_printf(const char *, ...);
extern int      tclio_fprintf(FILE *, const char *, ...);
extern int      tclio_fputs(const char *, FILE *);
extern char *tclio_fgets(char *, int, FILE *);
extern int      tclio_fputc(int, FILE *);
extern void     tclio_clearerr(FILE *);
extern int      tclio_fileno(FILE *);
extern int      tclio_ferror(FILE *);
extern int      tclio_fflush(FILE *);
extern FILE *tclio_fopen(const char *, const char *);
extern int      tclio_feof(FILE *);
extern int      tclio_fgetc(FILE *);
extern int      tclio_fungetc(const char,FILE *);
/*
extern char*	tclio_tmpnam(char *name);
extern FILE *	tclio_tmpfile(void);
*/
#ifdef __cplusplus
}
#endif

#endif  /* TCLSTDIO */
#endif  /* _TCLSTDIO_H */
